/*
    Exotic Pan Filter for VirtualDub.
	2017.02.16  Rename the "debug" to "Frame", put a keyboard interface on (F)rame
	2017.02.16  Only react on keyUp and repeat
	2017.02.14  Remove the "Dont crop" capability
	2017.01.06  Hiding PanelPoint.height and common pp "constructor"
	2017.01.06  Hiding PanelPoint.iOffset_y by set/get methods
	2017.01.06  Hiding PanelPoint.iOffset_x by set/get methods
	2017.01.04  Hiding PanelPoint.iFrame by set/get methods
	2016.12.21  Added keyboard handler for 'B* button. Changed 480p to 852 wide, which is 213*4
	2016.12.07  Added 540p button, added hook proc for keyboard input, move focus back to preview, modified RedoSystem to RedoFrame
	2016.07.17  Improved help load for 3 different plugin directory names.
	2016.07.13  Added 2 buttons. "Jump" which adds 2 rectangles to handle jump between scenes. & "Add N-1" copies last but one rectangel 
	2016.07.12  Fixup of central focus point to initially be centralized. also the "inside" button needed a fixup to work correctly. 
	2016.05.28  Bugfix Persistance method changed data in order to persist, so a subsequent load was required
*/


#include <windows.h>
#include <commctrl.h>
#include <crtdbg.h>
#include <assert.h>
#include <stdio.h>
#include <math.h>

#include "ScriptInterpreter.h"
#include "ScriptValue.h"
#include "ScriptError.h"

#include "resource.h"
#include "filter.h"
#include "InterModuleDef.h"


const
	int MOVE_SHIFT = 9;
	int MOVE_CTRL = 1;
	int MOVE_STD = 3;

static void HandleFieldOffsetX(HWND hDlg, UINT wParam, INT IDC_OFFSET_X, MyFilterData *mfd, UINT  xValOfs) {
	BOOL success;
	long focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X, &success, FALSE);
	if (HIWORD(wParam) == EN_UPDATE) {
		//if (!success || focusx > 1024) {
	}
	if (HIWORD(wParam) == EN_KILLFOCUS) {
		//mfd->PanelPointClosestToFrameIn
		if (!success || focusx > 2160) {
			SetDlgItemInt(hDlg, IDC_OFFSET_X, getOfsX(mfd, xValOfs), FALSE);
			return;
		}
		mfd->ifp->UndoSystem();
		setOfsX(mfd, xValOfs, (int) focusx);
		mfd->ifp->RedoSystem();
	}
}

static void HandleFieldOffsetY(HWND hDlg, UINT wParam, INT IDC_OFFSET_Y, MyFilterData *mfd, UINT  yValOfs) {
	BOOL success;
	long focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y, &success, FALSE);
	if (HIWORD(wParam) == EN_UPDATE) {
		//if (!success || focusx > 1024) {
	}
	if (HIWORD(wParam) == EN_KILLFOCUS) {
		if (!success || focusy > 2160) {
			SetDlgItemInt(hDlg, IDC_OFFSET_Y, getOfsY(mfd, yValOfs), FALSE);
			return;
		}
		mfd->ifp->UndoSystem();
		setOfsY(mfd, yValOfs, (int) focusy);
		mfd->ifp->RedoSystem();
	}
}

static void HandleFieldFrameNumber(HWND hDlg, UINT wParam, INT IDC_Frame, MyFilterData *mfd, UINT  frameValIdx) {

	if (HIWORD(wParam) == EN_KILLFOCUS) {
		BOOL success;
		long start = GetDlgItemInt(hDlg, IDC_Frame, &success, FALSE);
		if (!success) {
			SetDlgItemInt(hDlg, IDC_Frame,  getFrame(mfd, frameValIdx), FALSE);
			return;
		}
		mfd->ifp->UndoSystem();
		setFrame(mfd, frameValIdx, (int) start);
		if (bubbleSortIntervals(mfd->pp, mfd->iIntervals, INTERVALS)) {
			InitTableUI(hDlg, mfd);
		}
		mfd->ifp->RedoSystem();
	}
}

static void HandleFieldEnable(HWND hDlg, UINT wParam, INT IDC_ENABLED, MyFilterData *mfd, UINT  enabledValIdx) {
	if (HIWORD(wParam) == BN_CLICKED) {
		BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED);

		mfd->ifp->UndoSystem();
		mfd->pp[enabledValIdx].fEnabled = !!f;
		if (bubbleSortIntervals(mfd->pp, mfd->iIntervals, INTERVALS)) {
			InitTableUI(hDlg, mfd);
			postMessageIfClosestPanelChanges(mfd,  mfd->FrameOut);
		}
		mfd->ifp->RedoSystem();
	}
}

void SetFocusToPreview(HWND hDlg) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hDlg, DWL_USER);
	if (0 != mfd->preview) {
		SetActiveWindow(mfd->preview);
	}
}

void moveFrame(int dx, int dy, HWND hDlg, MyFilterData *mfd) {
	int idxFrame = mfd->PanelPointClosestToFrameIn;
	idxFrame = mfd->PanelPointClosestToFrameIn;  //This should be the one currently showing
	int val;
	if (dx != 0) {
		val = getOfsX(mfd, idxFrame) + dx * 5;
		setOfsX(mfd, idxFrame, val);
		SetDlgItemInt(hDlg, IDC_OFFSET_X_1, val, FALSE);
	}
	if (dy != 0) {
		val = getOfsY(mfd, idxFrame) + dy * 5;
		setOfsY(mfd, idxFrame, val);
		SetDlgItemInt(hDlg, IDC_OFFSET_Y_1, val, FALSE);
	}
	mfd->ifp->RedoFrame();
	SetFocusToPreview(hDlg); //if using the mouse on the a/w/s/d buttons in dialog
}

int getMyKeyState() {
	int iL = GetKeyState(VK_LSHIFT)>>1;
	int iR = GetKeyState(VK_RSHIFT)>>1;
 if (iL!=0 || iR!=0) return MOVE_SHIFT;

	iL = GetKeyState(VK_LCONTROL)>>1;
	iR = GetKeyState(VK_RCONTROL)>>1;

 if (iL!=0 || iR!=0) return MOVE_CTRL;

 return MOVE_STD;
}

void InitTableUI2(HWND hDlg, MyFilterData *mfd, int arrOfs) {
	for (int i = 0; i <3; i++) {
		if ((i+arrOfs-1>=0) && (i+arrOfs-1<=mfd->iIntervals) && mfd->pp[i+arrOfs-1].fEnabled) {
			SetDlgItemInt(hDlg, IDC_OFFSET_X_0 + i*4, getOfsX(mfd, i+arrOfs-1), FALSE);
			SetDlgItemInt(hDlg, IDC_OFFSET_Y_0 + i*4, getOfsY(mfd, i+arrOfs-1), FALSE);	
			SetDlgItemInt(hDlg, IDC_FRAME_0 + i*4,    getFrame(mfd, i+arrOfs-1),     FALSE);
		} else { //Blank the fields
			SetDlgItemInt(hDlg, IDC_OFFSET_X_0 + i*4, 0, FALSE);
			SetDlgItemInt(hDlg, IDC_OFFSET_Y_0 + i*4, 0, FALSE);	
			SetDlgItemInt(hDlg, IDC_FRAME_0 + i*4,    0, FALSE);
		}
	}
	#define eos(s) ((s)+strlen(s))
	char str[100]="";
	_snprintf(eos(str), 100,  "%d of %d",arrOfs, mfd->iIntervals);
	SetDlgItemText(hDlg, IDC_Key, str);  //Display the element number of this....
	UpdateUIRelatedToFrameChanges(hDlg, mfd);
}


void InitTableUI(HWND hDlg, MyFilterData *mfd) {
	int scrPos = mfd->PanelPointClosestToFrameIn;
	if (scrPos<0) scrPos = 0;

	HWND sbar = GetDlgItem(hDlg, IDS_SCL_CROP);
	SetScrollRange(sbar, SB_CTL, 0, mfd->iIntervals-1, false);
	SetScrollPos(sbar, SB_CTL, scrPos, true);

	InitTableUI2(hDlg, mfd, scrPos);
}

void UpdateUIRelatedToFrameChanges(HWND hDlg, MyFilterData *mfd) {
	SetDlgItemInt(hDlg, IDC_IN, mfd->FrameIn, FALSE);
	SetDlgItemInt(hDlg, IDC_OUT, mfd->FrameOut, FALSE);

	//Enable the "New" (Disable if exactly on top a a rectangle, as two rectangle with same definition doesnt make sense.
	int ppctfi = mfd->PanelPointClosestToFrameIn;
	bool bEnableBtn = ppctfi>=mfd->iIntervals || getFrame(mfd, ppctfi) != mfd->FrameIn;
	EnableWindow(GetDlgItem(hDlg, IDP_PB_NEW), bEnableBtn);

	//Enable together with New, provided at least two pptcfi exist.
	EnableWindow(GetDlgItem(hDlg, IDP_PB_AddBefore), (bEnableBtn & (ppctfi>=1)));
	if (bEnableBtn) { //Jump button is disabled together with add button + when current frame-1 is a ppctfi
		ppctfi = locatePanelPointClosestTOFrame(mfd, mfd->FrameIn-1, mfd->PanelPointClosestToFrameIn);
		bEnableBtn = ppctfi>=mfd->iIntervals || getFrame(mfd, ppctfi) != mfd->FrameIn-1;
	}
	EnableWindow(GetDlgItem(hDlg, IDP_PB_AddJump), bEnableBtn);
}

bool AddCurentPosToFrames(int frameNo, MyFilterData *mfd) {
	setPP(mfd, INTERVALS-1, frameNo, mfd->FocusX, mfd->FocusY, mfd->FocusH);
	return bubbleSortIntervals(mfd->pp, mfd->iIntervals, INTERVALS, true);
}

#define NUMHOOKS 1
// Global variables 
typedef struct _MYHOOKDATA 
{ 
	HWND hDlg;
    int nType; 
    HOOKPROC hkprc; 
    HHOOK hhook; 
} MYHOOKDATA; 
MYHOOKDATA myhookdata[NUMHOOKS]; 

LRESULT WINAPI BirdyKeyboardHookProc(int, WPARAM, LPARAM);

// https://msdn.microsoft.com/en-us/library/windows/desktop/ms644984(v=vs.85).aspx

LRESULT WINAPI BirdyKeyboardHookProc(int nType, WPARAM wParam, LPARAM lParam) {
	HWND hDlg = myhookdata[0].hDlg;
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hDlg, DWL_USER);

	if ((0xc000 & HIWORD(lParam)) !=0 ) {
		switch(LOWORD(wParam)) {
			case 'W':	moveFrame(0,getMyKeyState(), hDlg, mfd);	return true; /*up*/
			case 'A':	moveFrame(getMyKeyState(), 0, hDlg, mfd);	return true; /*left*/
			case 'S':	moveFrame(0,-getMyKeyState(), hDlg, mfd);	return true; /*down*/
			case 'D':	moveFrame(-getMyKeyState(), 0, hDlg, mfd);	return true; /*right*/
			case VK_DELETE:return SendMessage(hDlg,WM_COMMAND, IDP_PB_DEL, 0);
			case 'N':	return SendMessage(hDlg,WM_COMMAND, IDP_PB_NEW, 0);
			case 'J':	return SendMessage(hDlg,WM_COMMAND, IDP_PB_AddJump, 0);
			case 'B':	return SendMessage(hDlg,WM_COMMAND, IDP_PB_AddBefore, 0);
			case 'F':	//Toggle the checkbox, then call the hander.
				CheckDlgButton(hDlg, IDC_SHOW_FRAME, IsDlgButtonChecked(hDlg, IDC_SHOW_FRAME) ? BST_UNCHECKED : BST_CHECKED);
				return SendMessage(hDlg,WM_COMMAND, IDC_SHOW_FRAME, 0);
			case 'I':	return SendMessage(hDlg,WM_COMMAND, IDP_PB_Inside, 0);
		}
	} else {
		switch(LOWORD(wParam)) {
			case 'W':
			case 'A':
			case 'S':
			case 'D':
			case VK_DELETE:
			case 'N':
			case 'J':
			case 'B':
			case 'F':
			case 'I': return 1;
		}
	}
	return CallNextHookEx(myhookdata[0].hhook, nType , wParam, lParam); 
}

void RemoveHooks() {
	if (0!=myhookdata[0].hhook) {
		UnhookWindowsHookEx(myhookdata[0].hhook);
		myhookdata[0].hhook = 0;
	} 
}

BOOL APIENTRY resizeDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam) {
	MyFilterData *mfd = (MyFilterData *)GetWindowLong(hDlg, DWL_USER);

	LRESULT lresult = NULL;
    switch (message)
    {
        case WM_INITDIALOG:
			{
				mfd = (MyFilterData *)lParam;
				mfd->hDlg = hDlg;  //Save window proc handle so frame numbers can be cummunicated.
				SetDlgItemInt(hDlg, IDC_WIDTH, mfd->new_x, FALSE);
				SetDlgItemInt(hDlg, IDC_HEIGHT, mfd->new_y, FALSE);

				InitTableUI(hDlg, mfd);

				SetDlgItemInt(hDlg, IDC_FILLRED, mfd->red, FALSE);
				SetDlgItemInt(hDlg, IDC_FILLGREEN, mfd->green, FALSE);
				SetDlgItemInt(hDlg, IDC_FILLBLUE, mfd->blue, FALSE);

				CheckDlgButton(hDlg, IDC_SHOW_FRAME, mfd->fDebug ? BST_CHECKED : BST_UNCHECKED);

				SetWindowLong(hDlg, DWL_USER, (LONG)mfd);
				mfd->ifp->InitButton(GetDlgItem(hDlg, IDC_PREVIEW));

				SendMessage(mfd->hDlg, WM_USER + WM_USER_UPDATE_FRAME_NO, 0, 0); //Tell DLG to update index of last processed frame.

				myhookdata[0].nType = WH_KEYBOARD;
				myhookdata[0].hkprc = BirdyKeyboardHookProc; 
				myhookdata[0].hDlg = hDlg;

			}
            return (TRUE);

		case WM_KEYDOWN:
			return (FALSE);

        case WM_COMMAND:                      
			if (mfd == NULL)
				return TRUE; //Nothing works in this situation.

			switch(LOWORD(wParam)) {
			case IDOK:
				RemoveHooks();
				mfd->ifp->Close();
				mfd->hDlg = 0;
				EndDialog(hDlg, 0);
				mfd->fDebug = false;
				return TRUE;

			case IDCANCEL:
				RemoveHooks();
				mfd->ifp->Close();
				mfd->hDlg = 0;
                EndDialog(hDlg, 1);
				mfd->fDebug = false;
                return TRUE;

			case IDC_WIDTH:
				if (HIWORD(wParam) == EN_KILLFOCUS) {
					BOOL success;
					long new_x = GetDlgItemInt(hDlg, IDC_WIDTH, &success, FALSE);
					if (!success || new_x < 1 || new_x > 3840) {  //4K is 1920 * 2
						SetDlgItemInt(hDlg, IDC_WIDTH, mfd->new_x, FALSE);
						return TRUE;
					}
					mfd->ifp->UndoSystem();
					mfd->new_x = new_x;
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_HEIGHT:
				if (HIWORD(wParam) == EN_KILLFOCUS) {
					BOOL success;
					long new_y = GetDlgItemInt(hDlg, IDC_HEIGHT, &success, FALSE);
					if (!success || new_y < 1 || new_y > 2160) {  //4k is 1080 * 2 i think
						SetDlgItemInt(hDlg, IDC_HEIGHT, mfd->new_y, FALSE);
						return TRUE;
					}
					mfd->ifp->UndoSystem();
					mfd->new_y = new_y;
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 1 *******************************
			case IDC_OFFSET_X_1:
				HandleFieldOffsetX(hDlg, wParam, IDC_OFFSET_X_1, mfd, mfd->PanelPointClosestToFrameIn);
				return TRUE;

			case IDC_OFFSET_Y_1:
				HandleFieldOffsetY(hDlg, wParam, IDC_OFFSET_Y_1, mfd, mfd->PanelPointClosestToFrameIn);
				return TRUE;

			case IDC_FRAME_1:
				HandleFieldFrameNumber(hDlg, wParam, IDC_FRAME_1, mfd, mfd->PanelPointClosestToFrameIn);
				return TRUE;

			case IDC_ENABLED_1:
				HandleFieldEnable(hDlg, wParam, IDC_ENABLED_1, mfd, mfd->PanelPointClosestToFrameIn);
				return TRUE;

			// ************
			case IDC_FILLRED:
				if (HIWORD(wParam) == EN_KILLFOCUS) {
					BOOL success;
					long red = GetDlgItemInt(hDlg, IDC_FILLRED, &success, FALSE);
					if (!success || red > 255) {
						SetDlgItemInt(hDlg, IDC_FILLRED, mfd->red, FALSE);
						return TRUE;
					}
					mfd->ifp->UndoSystem();
					mfd->red = red;
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_FILLGREEN:
				if (HIWORD(wParam) == EN_UPDATE) {
					BOOL success;
					long green = GetDlgItemInt(hDlg, IDC_FILLGREEN, &success, FALSE);
					if (!success || green > 255) {
						SetDlgItemInt(hDlg, IDC_FILLGREEN, mfd->green, FALSE);
						return TRUE;
					}
					mfd->ifp->UndoSystem();
					mfd->green = green;
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_FILLBLUE:
				if (HIWORD(wParam) == EN_KILLFOCUS) {
					BOOL success;
					long blue = GetDlgItemInt(hDlg, IDC_FILLBLUE, &success, FALSE);
					if (!success || blue > 255) {
						SetDlgItemInt(hDlg, IDC_FILLBLUE, mfd->blue, FALSE);
						return TRUE;
					}
					mfd->ifp->UndoSystem();
					mfd->blue = blue;
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_PREVIEW:
				mfd->ifp->Toggle(hDlg);
				mfd->preview = FindWindow(NULL, "Filter preview");
				mfd->birdy = FindWindowEx(mfd->preview, NULL, "birdyPositionControl", NULL);
				if (0==mfd->preview) {
					RemoveHooks();
				} else {
					myhookdata[0].hhook = SetWindowsHookEx(myhookdata[0].nType, myhookdata[0].hkprc, (HINSTANCE) NULL, GetCurrentThreadId()); 
				}
				return TRUE;

			case IDC_SHOW_FRAME:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_SHOW_FRAME);

					mfd->fDebug = !!f;
					mfd->ifp->RedoFrame(); //the rectangle neds repainting.
					SetFocusToPreview(hDlg);
				}
				return TRUE;

 			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;
				HINSTANCE retCode;

				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat_s(path, "plugins\\ExoticPan.html");
				retCode = ShellExecute(hDlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				if (0 != (int)retCode) {
					GetFullPathName(prog, 255, path, &ptr);
					*ptr = 0;
					strcat_s(path, "plugins32\\ExoticPan.html");
					retCode = ShellExecute(hDlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				}
				if (0 != (int)retCode) {
					GetFullPathName(prog, 255, path, &ptr);
					*ptr = 0;
					strcat_s(path, "plugins64\\ExoticPan.html");
					retCode = ShellExecute(hDlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				}
				if (0 != (int)retCode) {
					GetFullPathName(prog, 255, path, &ptr);
					*ptr = 0;
					MessageBox(hDlg,"Attempted to load \"ExoticPan.html\" from \n\tplugins, \n\tplugins32,\n\tplugins64\n in the path path shown in title",path,MB_OK | MB_ICONWARNING);
				}
				return TRUE;
				}

			case IDP_720p:
				{
					mfd->ifp->UndoSystem();
					SetDlgItemInt(hDlg, IDC_WIDTH, 1280, FALSE);  //1280 x 720
					SetDlgItemInt(hDlg, IDC_HEIGHT, 720, FALSE);  
					mfd->new_x = 1280;
					mfd->new_y = 720;
					mfd->ifp->RedoSystem();
					return TRUE;
				}

			case IDP_576p:
				{
					mfd->ifp->UndoSystem();
					SetDlgItemInt(hDlg, IDC_WIDTH, 1024, FALSE);  //720 x 576
					SetDlgItemInt(hDlg, IDC_HEIGHT, 576, FALSE);  
					mfd->new_x = 1024;
					mfd->new_y = 576;
					mfd->ifp->RedoSystem();
					return TRUE;
				}

			case IDP_540p:
				{
					mfd->ifp->UndoSystem();
					SetDlgItemInt(hDlg, IDC_WIDTH, 960, FALSE);  //720 x 576
					SetDlgItemInt(hDlg, IDC_HEIGHT, 540, FALSE);  
					mfd->new_x = 960;
					mfd->new_y = 540;
					mfd->ifp->RedoSystem();
					return TRUE;
				}

			case IDP_480p:
				{
					long new_x;
					BOOL success; //852 / 640 x 480

					mfd->ifp->UndoSystem();
					new_x = GetDlgItemInt(hDlg, IDC_WIDTH, &success, FALSE);
					if (!success || new_x != 854 || new_x > 3840) {
						SetDlgItemInt(hDlg, IDC_WIDTH, 852, FALSE);  
						mfd->new_x = 852;
					} else {
						SetDlgItemInt(hDlg, IDC_WIDTH, 640, FALSE);  
						mfd->new_x = 640;
					}
					SetDlgItemInt(hDlg, IDC_HEIGHT, 480, FALSE);  
					mfd->new_y = 480;
					mfd->ifp->RedoSystem();
					return TRUE;
				}

			case IDP_360p:
				{
					long new_x;
					BOOL success; //640 / 480 x 360

					new_x = GetDlgItemInt(hDlg, IDC_WIDTH, &success, FALSE);
					if (!success || new_x != 640 || new_x > 3840) {  
						SetDlgItemInt(hDlg, IDC_WIDTH, 640, FALSE);  
						mfd->new_x = 640;
					} else {
						SetDlgItemInt(hDlg, IDC_WIDTH, 480, FALSE);  
						mfd->new_x = 480;
					}
					SetDlgItemInt(hDlg, IDC_HEIGHT, 360, FALSE);  
					mfd->new_y = 360;
					mfd->ifp->RedoSystem();
					return TRUE;
				}
			case IDP_PB_NEW:
				{
					//Initial strategy. 
					//1 Put the current (calculated) offsets into last eleent, 
					//2 activate it. 
					//3 call resort, and a repaint of the dialoge.
					if (IsWindowEnabled(GetDlgItem(hDlg, IDP_PB_NEW))) { //Keyboard actions as well
						AddCurentPosToFrames(mfd->FrameIn, mfd);
						InitTableUI(hDlg, mfd);
						postMessageIfClosestPanelChanges(mfd,  mfd->FrameOut);
						SetFocusToPreview(hDlg);
					}
				}
				return TRUE;

			case IDP_PB_AddJump:
				if (IsWindowEnabled(GetDlgItem(hDlg, IDP_PB_AddJump))) { //Keyboard actions as well
					AddCurentPosToFrames(mfd->FrameIn-1, mfd);
					AddCurentPosToFrames(mfd->FrameIn, mfd);
					InitTableUI(hDlg, mfd);
					postMessageIfClosestPanelChanges(mfd,  mfd->FrameOut);
					SetFocusToPreview(hDlg);
				}
				return TRUE;

			case IDP_PB_AddBefore:
				{
					int ppctfi = mfd->PanelPointClosestToFrameIn;
					if (getFrame(mfd, ppctfi) > mfd->FrameIn) ppctfi--; //In case the nearest is after current position
					if (ppctfi>=1) {
						//Step back to panel before this one and add it.
						ppctfi--;

						setPP(mfd, INTERVALS-1, mfd->FrameIn, getOfsX(mfd, ppctfi), getOfsY(mfd, ppctfi), getHeight(mfd, ppctfi));
						bubbleSortIntervals(mfd->pp, mfd->iIntervals, INTERVALS, true);
						InitTableUI(hDlg, mfd);
						postMessageIfClosestPanelChanges(mfd,  mfd->FrameOut);
						mfd->ifp->RedoSystem(); //Have copied naoter rectangle def. Better redraw.
					}
					SetFocusToPreview(hDlg);
					return TRUE;
				}

			case IDP_PB_DEL:
				if (IsWindowEnabled(GetDlgItem(hDlg, IDP_PB_DEL))) {
					mfd->pp[mfd->PanelPointClosestToFrameIn].fEnabled = false;
					bubbleSortIntervals(mfd->pp, mfd->iIntervals, INTERVALS, true);
					InitTableUI(hDlg, mfd);
					postMessageIfClosestPanelChanges(mfd,  mfd->FrameOut);
					mfd->ifp->RedoSystem(); //This delete may move the rectangle
					SetFocusToPreview(hDlg);
				}
				return TRUE;
					
			case IDP_PB_Inside: {
				//Check if selected rectange is widthin borders.
				bool bAdjusted = false; 
				PanelPoint *pFocused = &mfd->pp[mfd->PanelPointClosestToFrameIn];
				if (getOfsX(pFocused)  < mfd->new_x/2) {
					setOfsX(pFocused, mfd->new_x/2);
					bAdjusted = true;
				}
				if (getOfsY(pFocused) < mfd->new_y/2) {
					setOfsY(pFocused, mfd->new_y/2);
					bAdjusted = true;
				}
				if (getOfsX(pFocused) >= mfd->src_w - mfd->new_x/2) {
					setOfsX(pFocused, mfd->src_w - mfd->new_x/2 - 1);
					bAdjusted = true;
				}
				if (getOfsY(pFocused) >= mfd->src_h - mfd->new_y/2) {
					setOfsY(pFocused, mfd->src_h - mfd->new_y/2 - 1);
					bAdjusted = true;
				}
				if (bAdjusted) {
					InitTableUI(hDlg, mfd);
					postMessageIfClosestPanelChanges(mfd,  mfd->FrameOut);
					mfd->ifp->RedoFrame(); //This may move the rectangle
				}
				SetFocusToPreview(hDlg);
			}
			return TRUE;

			case IDP_CropUp_W:	 moveFrame(0,getMyKeyState(), hDlg, mfd);	return true; /*up*/
			case IDP_CropLeft_A: moveFrame(getMyKeyState(), 0, hDlg, mfd);	return true; /*left*/
			case IDP_CropDown_S: moveFrame(0,-getMyKeyState(), hDlg, mfd);	return true; /*down*/
			case IDP_CropRight_D:moveFrame(-getMyKeyState(), 0, hDlg, mfd);	return true; /*right*/

			case IDS_SCL_CROP:
				{
					return TRUE;
				}
			}
            break;
		case WM_VSCROLL:{
			HWND hScl = (HWND) lParam;
			SCROLLINFO si;
			ZeroMemory(&si, sizeof(si));
			si.cbSize = sizeof(si);
			si.fMask = SIF_TRACKPOS;

			GetScrollInfo(hScl, SB_CTL,  &si);  //SB_CTL SB_HORZ SB_VERT
			GetScrollInfo(hScl, SB_HORZ, &si);  //SB_CTL SB_HORZ SB_VERT
			GetScrollInfo(hScl, SB_VERT, &si);  //SB_CTL SB_HORZ SB_VERT
			switch(LOWORD(wParam)) {
				case SB_THUMBTRACK:
				{

					return TRUE;
				}
			} 
			return true;
		}

		case WM_HSCROLL:
				{
					return TRUE;
				}

		case WM_USER + WM_USER_UPDATE_FOCUSED_PP:
				InitTableUI(hDlg, mfd);
			break;
		case WM_USER + WM_USER_UPDATE_FRAME_NO:
			UpdateUIRelatedToFrameChanges(hDlg, mfd);
		break;


    }
    return FALSE;
}
