/*
    Exotic Pan Filter for VirtualDub.
	2017.02.14  Remove the "Dont crop" capability
	2017.01.06  Hiding PanelPoint.height and common pp "constructor"
	2017.01.06  Hiding PanelPoint.iOffset_y by set/get methods
	2017.01.06  Hiding PanelPoint.iOffset_x by set/get methods
	2017.01.04  Hiding PanelPoint.iFrame by set/get methods
	2016.12.07  added hoop proc for keyboard input
	2016.05.28  Bugfix Persistance method changed data in order to persist, so a subsequent load was required
*/


const int INTERVALS = 100;



typedef struct PanelPoint {
	int iFrame;   //Moved to top to minimize risk if frame or offfset is written by 4 byte int
	int iOffset_x;
	int iOffset_y;
	int iHeight; //Height of selection 0 means not used (Indirectly zoom factor)
	bool fEnabled;
} PanelPoint;

////////////////////

typedef struct MyFilterData {
	long new_x;  //Width of new/output frame
	long new_y;  //Height of new/output frame
	long src_w;  //Width of new/output frame
	long src_h;  //Height of new/output frame

	PanelPoint pp [INTERVALS]; //Arrray of pan definitions
	int iIntervals; //# of panelpoints in use.
	
	// other
	int red, green, blue;
	bool fDebug;
	HWND preview;
	HWND birdy;

	IFilterPreview *ifp;
	int FrameIn, FrameOut; //Frame numbers communicated to dialog.
	int FocusX, FocusY, FocusH;
	int PanelPointClosestToFrameIn;
	HWND hDlg;  //0 except when DLG is active.
} MyFilterData;

////////////////////
BOOL APIENTRY resizeDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam);
void UpdateUIRelatedToFrameChanges(HWND hDlg, MyFilterData *mfd);
void InitTableUI(HWND hDlg, MyFilterData *mfd);
bool bubbleSortIntervals(struct PanelPoint pp[], int &iIntervals, int INTERVALS, bool bChangedAnything);
bool bubbleSortIntervals(struct PanelPoint pp[], int &iIntervals, int INTERVALS);
void postMessageIfClosestPanelChanges(MyFilterData *mfd, int frame);
int locatePanelPointClosestTOFrame(MyFilterData *mfd, int frame, int seed);

int getFrame(MyFilterData * mfd, int idx);
int getFrame(PanelPoint pp[], int idx);
void setFrame(MyFilterData * mfd, int idx, int value);

int getOfsX(MyFilterData * mfd, int idx);
int getOfsX(PanelPoint pp[], int idx);
int getOfsX(PanelPoint * pp);
void setOfsX(MyFilterData * mfd, int idx, int value);
void setOfsX(PanelPoint pp[], int idx, int value);
void setOfsX(PanelPoint * pp, int value);

int getOfsY(PanelPoint* pp);
int getOfsY(MyFilterData * mfd, int idx);
void setOfsY(PanelPoint* pp, int value);
void setOfsY(MyFilterData * mfd, int idx, int value);

int getHeight(MyFilterData * mfd, int idx);
void setHeight(MyFilterData * mfd, int idx, int value);

void setPP(PanelPoint* pp, int Frame, int ofsX, int ofsY, int height);
void setPP(MyFilterData * mfd, int idx, int frame, int ofsX, int ofsY, int height);

void swapFrames(PanelPoint pp[], int a, int b);
